// --------------------------------------------------------------------
// Copyright (c) 2012 by Terasic Technologies Inc.
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// --------------------------------------------------------------------
//
//                     Terasic Technologies Inc
//                     E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------

#include "terasic_includes.h"
#include "clock.h"
#include "si5340A_config.h"
#include "si5340B_config.h"
#include "in231.h"


typedef bool (*LP_VERIFY_FUNC)(void);


bool TEST_Temperature(void);
bool TEST_Power_Monitor(void);
bool TEST_Si5340A(void);
bool TEST_Si5340B(void);


typedef struct{
    LP_VERIFY_FUNC func;
    char szName[128];
}FUNC_INFO;

FUNC_INFO szFuncList[] = {
    {TEST_Temperature, "Temperature"}, //ok
    {TEST_Power_Monitor, "Power_Monitor"}, //ok
    {TEST_Si5340A, "Si5340A"}, // ok
    {TEST_Si5340B, "Si5340B"} // ok
};


void GUI_ShowMenu(void){
    int nNum,i;
    
    nNum = sizeof(szFuncList)/sizeof(szFuncList[0]);
    printf("======= Arria 10 Demo Program =======\r\n");
    for(i=0;i<nNum;i++){
        printf("[%d] %s\r\n", i, szFuncList[i].szName);
    }
    printf("Input your chioce:");
}

int GUI_QueryUser(void){
    int nChoice = 0;
    scanf("%d", &nChoice);
    printf("%d\r\n", nChoice);
    return nChoice;
}

//===============================================================
int main(void){
    int nChoice;
    int nNum;
    bool bPass;
    
    nNum = sizeof(szFuncList)/sizeof(szFuncList[0]);
    while(1){
    	GUI_ShowMenu();
        nChoice = GUI_QueryUser();
        if (nChoice >= 0 && nChoice < nNum){
            bPass = szFuncList[nChoice].func();
            printf("%s Test:%s\r\n", szFuncList[nChoice].szName, bPass?"PASS":"NG");
        }            
    }     
    
}


bool TEST_Temperature(void){
    bool bPass = FALSE;
    const alt_u8 DeviceAddr = 0x30;
    alt_u8 LocalTemp, RemoteTemp;

    bPass = I2C_Read(TEMP_SCL_BASE, TEMP_SDA_BASE, DeviceAddr, 0x00, &LocalTemp);
    if (bPass)
        bPass = I2C_Read(TEMP_SCL_BASE, TEMP_SDA_BASE, DeviceAddr, 0x01, &RemoteTemp);

    if (bPass){
        printf("Local Temperature:%d\r\n", (char)LocalTemp);
        printf("Remote Temperature:%d\r\n", (char)RemoteTemp);
    }else{
        printf("Failed to read temperature\r\n");
    }        
    return bPass;
}

bool TEST_Power_Monitor(void)
{
   bool bPass ;
   alt_u16 data;
   float Shunt_Voltage,Bus_Voltage,Current,Power;
   //Configuration Register
   bPass = I2C_Write16(PMONITOR_I2C_SCL_BASE,PMONITOR_I2C_SDA_BASE,Monitor_I2C_ADRESS,P_Configuration,Configuration_Reg);
   if(bPass){
	         bPass = I2C_Write16(PMONITOR_I2C_SCL_BASE,PMONITOR_I2C_SDA_BASE,Monitor_I2C_ADRESS,P_Calibration,Calibration_Reg);
	         if(bPass){
	          		 printf("Configuration ok! \n");
	         }else{
	          		 printf("Configuration failed!  \n");
	         }
   }else{
    		printf("Configuration failed!  \n");
   }
   usleep(2000);

   printf("==== Power Monitor Test ====\r\n");
   // Read Shunt_Voltage
   bPass = I2C_Read16(PMONITOR_I2C_SCL_BASE,PMONITOR_I2C_SDA_BASE,Monitor_I2C_ADRESS,P_Shunt_Voltage,&data);
   if(bPass){
	   	   	Shunt_Voltage = (float)data*2.5/1000;
	   	   	printf("Shunt_Voltage = %.3f mV \n",Shunt_Voltage);
   }else{
   	        printf("Read Shunt_Voltage failed!\n");
   }
   // Read Bus_Voltage
   bPass = I2C_Read16(PMONITOR_I2C_SCL_BASE,PMONITOR_I2C_SDA_BASE,Monitor_I2C_ADRESS,P_Bus_Voltage,&data);
   if(bPass){
        	Bus_Voltage = (float)data*1.25/1000;
        	printf("Bus_Voltage   = %.3f V \n",Bus_Voltage);
   }else{
    	    printf("Read Bus_Voltage failed!\n");
   }
   //Read Current
   bPass = I2C_Read16(PMONITOR_I2C_SCL_BASE,PMONITOR_I2C_SDA_BASE,Monitor_I2C_ADRESS,P_Current,&data);
   if(bPass){
    	    Current = (float)data*0.5/1000;
    	    printf("Current       = %.3f A \n",Current);
   }else{
    	    printf("Read Current failed!\n");
   }
   //Read Power
   bPass = I2C_Read16(PMONITOR_I2C_SCL_BASE,PMONITOR_I2C_SDA_BASE,Monitor_I2C_ADRESS,P_Power,&data);
   if(bPass){
        	Power = (float)data*0.5*25/1000;
        	printf("Power         = %.3f W \n",Power);
   }else{
    	    printf("Read Power failed!\n");
   }
   return bPass;

 }


bool TEST_Si5340A(void){
	typedef struct{
		int FreqID;
		float FREQUENCY;
	}Si5340A_CONFIG;

	Si5340A_CONFIG szConfig[] = {
		{SI5340_100M, 100.0},
		{SI5340_125M, 125.0},
		{SI5340_156M25, 156.25},
		{SI5340_250M, 250.0},
		{SI5340_312M5, 312.5},
		{SI5340_322M265625, 322.265625},
		{SI5340_644M53125, 644.53125},
		{Not_Use, 0.0}
	};
	bool bPass;
	alt_u32 clk1, clk2,ScaledClk2;
	const int nTargetCnt=1000;
	const int nTol = 3;  //tolerance
	int i, j, k, m, nNum;
	nNum = sizeof(szConfig)/sizeof(szConfig[0]);
	
	// show menu
	printf("===== Si5340A Programming =====\r\n");
    IOWR(SI5340A_OE_N_BASE, 0, 0x00);
    IOWR(SI5340A_RST_BASE, 0, 0x00);
    usleep(1000);
    IOWR(SI5340A_RST_BASE, 0, 0x01);
    usleep(1000);
	for(i=0;i<nNum;i++){
		printf("[%d] %.6f MHz\r\n",i, szConfig[i].FREQUENCY);
	}
	printf("[Other] exit\r\n");
	printf("please select QSFPA_REFCLK:");
	i = GUI_QueryUser();
	printf("please select QSFPB_REFCLK:");
	j = GUI_QueryUser();
	printf("please select QSFPC_REFCLK:");
	k = GUI_QueryUser();
	printf("please select QSFPD_REFCLK:");
	m = GUI_QueryUser();
	if ((i> nNum) || (j> nNum) || (k> nNum) || (m> nNum))
		return FALSE;
	usleep(100*1000);
	bPass = SI5340A_Config_Init();
	if(bPass){
		bPass = SI5340A_Config(szConfig[i].FreqID, szConfig[j].FreqID,szConfig[k].FreqID,szConfig[m].FreqID);
	}

	if (bPass){
		//wait
		usleep(200*1000);
		// measure
		bPass =  CLOCK_Test(REF_CLOCK_QSFPA_COUNT_BASE, nTargetCnt, &clk1, &clk2);
		if (bPass){
			ScaledClk2 = (i==7) ? clk1 : (int)((float) clk2 * 50.0 / szConfig[i].FREQUENCY);
			if (abs(nTargetCnt-clk1)<=nTol && abs(clk1-ScaledClk2)<=nTol){
				printf("QSFPA/%.6fMHz ref clock test PASS (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[i].FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[i].FREQUENCY/50.0));
			}else{
				printf("QSFPA/%.6fMHz ref clock test NG (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[i].FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[i].FREQUENCY/50.0));
				bPass = FALSE;
			}
			}else{
				printf("Failed to perform CLOCK_Test\r\n");
			}

	}
	if (bPass){
		//wait
		usleep(200*1000);
		// measure
		bPass =  CLOCK_Test(REF_CLOCK_QSFPB_COUNT_BASE, nTargetCnt, &clk1, &clk2);
		if (bPass){
			ScaledClk2 = (j==7) ? clk1 :  (int)((float) clk2 * 50.0 / szConfig[j].FREQUENCY);
			if (abs(nTargetCnt-clk1)<=nTol && abs(clk1-ScaledClk2)<=nTol){
				printf("QSFPB/%.6fMHz ref clock test PASS (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[j].FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[j].FREQUENCY/50.0));
			}else{
				printf("QSFPB/%.6fMHz ref clock test NG (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[j].FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[j].FREQUENCY/50.0));
				bPass = FALSE;
			}
			}else{
				printf("Failed to perform CLOCK_Test\r\n");
			}

	}
	if (bPass){
		//wait
		usleep(200*1000);
		// measure
		bPass =  CLOCK_Test(REF_CLOCK_QSFPC_COUNT_BASE, nTargetCnt, &clk1, &clk2);
		if (bPass){
			ScaledClk2 = (k==7) ? clk1 : (int)((float) clk2 * 50.0 / szConfig[k].FREQUENCY);
			if (abs(nTargetCnt-clk1)<=nTol && abs(clk1-ScaledClk2)<=nTol){
				printf("QSFPC/%.6fMHz ref clock test PASS (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[k].FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[k].FREQUENCY/50.0));
			}else{
				printf("QSFPC/%.6fMHz ref clock test NG (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[k].FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[k].FREQUENCY/50.0));
				bPass = FALSE;
			}
			}else{
				printf("Failed to perform CLOCK_Test\r\n");
			}

	}
	if (bPass){
		//wait
		usleep(200*1000);
		// measure
		bPass =  CLOCK_Test(REF_CLOCK_QSFPD_COUNT_BASE, nTargetCnt, &clk1, &clk2);
		if (bPass){
			ScaledClk2 = (m==7) ? clk1 : (int)((float) clk2 * 50.0 / szConfig[m].FREQUENCY);
			if (abs(nTargetCnt-clk1)<=nTol && abs(clk1-ScaledClk2)<=nTol){
				printf("QSFPD/%.6fMHz ref clock test PASS (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[m].FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[m].FREQUENCY/50.0));
			}else{
				printf("QSFPD/%.6fMHz ref clock test NG (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[m].FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[m].FREQUENCY/50.0));
				bPass = FALSE;
			}
			}else{
				printf("Failed to perform CLOCK_Test\r\n");
			}

	}

return bPass;

}




bool TEST_Si5340B(void){
	typedef struct{
		int FreqID;
		float DDR3_FREQUENCY;
		float PCIE_FREQUENCY;
		float QDRII_FREQUENCY;
	}Si5340B_CONFIG;

	Si5340B_CONFIG szConfig[] = {
		{SI5340_CASE0,  266.667,100,275},
		{SI5340_CASE1,  233.333,100,275},
		{SI5340_CASE2,  200,100,275},
		{SI5340_CASE3,  266.667,100,250},
		{SI5340_CASE4,  233.333,100,250},
		{SI5340_CASE5,  200,100,250},
		{SI5340_CASE6,  266.667,100,225},
		{SI5340_CASE7,  233.333,100,225},
		{SI5340_CASE8,  200,100,225},
		{SI5340_CASE9,  266.667,100,233.333},
		{SI5340_CASE10, 233.333,100,233.333},
		{SI5340_CASE11, 200,100,233.333},
	};
	bool bPass;
	alt_u32 clk1, clk2,ScaledClk2;
	const int nTargetCnt=1000;
	const int nTol = 3;  //tolerance
	int i,nNum;
	nNum = sizeof(szConfig)/sizeof(szConfig[0]);

	// show menu
	printf("===== Si5340B Programming =====\r\n");
    IOWR(SI5340B_OE_N_BASE, 0, 0x00);
    IOWR(SI5340B_RST_BASE, 0, 0x00);
    usleep(1000);
    IOWR(SI5340B_RST_BASE, 0, 0x01);
    usleep(1000);
	for(i=0;i<nNum;i++){
		printf("[%d] DDR3:%.6f MHz PCIE:%.6f MHz QDRII:%.6f MHz\r\n",i, szConfig[i].DDR3_FREQUENCY,szConfig[i].PCIE_FREQUENCY,szConfig[i].QDRII_FREQUENCY);
	}
	printf("[Other] exit\r\n");
	printf("please select SI5340B_CASE:");
	i = GUI_QueryUser();

	if (i> nNum)
		return FALSE;
	bPass = SI5340B_Config_Init();
	if(bPass){
		bPass = SI5340B_Config(szConfig[i].FreqID);
	}

	if (bPass){
		//wait
		usleep(200*1000);
		// measure
		bPass =  CLOCK_Test(REF_CLOCK_DDR3_COUNT_BASE, nTargetCnt, &clk1, &clk2);
		if (bPass){
			ScaledClk2 = (int)((float) clk2 * 50.0 / szConfig[i].DDR3_FREQUENCY);
			if (abs(nTargetCnt-clk1)<=nTol && abs(clk1-ScaledClk2)<=nTol){
				printf("DDR3/%.6fMHz ref clock test PASS (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[i].DDR3_FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[i].DDR3_FREQUENCY/50.0));
			}else{
				printf("DDR3/%.6fMHz ref clock test NG (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[i].DDR3_FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[i].DDR3_FREQUENCY/50.0));
				bPass = FALSE;
			}
			}else{
				printf("Failed to perform CLOCK_Test\r\n");
			}

	}
	if (bPass){
		//wait
		usleep(200*1000);
		// measure
		bPass =  CLOCK_Test(REF_CLOCK_PCIE_COUNT_BASE, nTargetCnt, &clk1, &clk2);
		if (bPass){
			ScaledClk2 = (int)((float) clk2 * 50.0 / szConfig[i].PCIE_FREQUENCY);
			if (abs(nTargetCnt-clk1)<=nTol && abs(clk1-ScaledClk2)<=nTol){
				printf("PCIE/%.6fMHz ref clock test PASS (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[i].PCIE_FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[i].PCIE_FREQUENCY/50.0));
			}else{
				printf("PCIE/%.6fMHz ref clock test NG (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[i].PCIE_FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[i].PCIE_FREQUENCY/50.0));
				bPass = FALSE;
			}
			}else{
				printf("Failed to perform CLOCK_Test\r\n");
			}

	}

	if (bPass){
		//wait
		usleep(200*1000);
		// measure
		bPass =  CLOCK_Test(REF_CLOCK_QDRII_COUNT_BASE, nTargetCnt, &clk1, &clk2);
		if (bPass){
			ScaledClk2 = (int)((float) clk2 * 50.0 / szConfig[i].QDRII_FREQUENCY);
			if (abs(nTargetCnt-clk1)<=nTol && abs(clk1-ScaledClk2)<=nTol){
				printf("QDRII/%.6fMHz ref clock test PASS (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[i].QDRII_FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[i].QDRII_FREQUENCY/50.0));
			}else{
				printf("QDRII/%.6fMHz ref clock test NG (clk1=%d, clk2=%d, expected clk2=%d)\r\n",  szConfig[i].QDRII_FREQUENCY, (int)clk1, (int)clk2, (int)((float)clk1*szConfig[i].QDRII_FREQUENCY/50.0));
				bPass = FALSE;
			}
			}else{
				printf("Failed to perform CLOCK_Test\r\n");
			}

	}

return bPass;

}



