// --------------------------------------------------------------------
// Copyright (c) 2012 by Terasic Technologies Inc.
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// --------------------------------------------------------------------
//
//                     Terasic Technologies Inc
//                     E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------

#include "terasic_includes.h"
#include "mem_verify.h"


//#define STATUS_BIT_DONE 	0x01
#define STATUS_BIT_FAIL 	0x02
#define STATUS_BIT_SUCCESS 	0x01
#define A_BIT_MASK	0x03

#define B_STATUS_BIT_FAIL 	0x08
#define B_STATUS_BIT_SUCCESS 	0x04
#define B_BIT_MASK	0x0C




int main()
{
	bool bPassA, bPassB, bLoop;
	int TestIndex ;
	alt_u32 InitValue = 0x01;
	bool bShowMessage = TRUE, bTimeout, bDone;
	alt_u32 TimeStart, TimeElapsed;
	alt_u8 Status,ButtonStatus;
	const alt_u8 ButtonMask = 0x0F; // 4 button
	const alt_u32 CalTimeout = alt_ticks_per_second();

	IOWR(REQEUST_DDR3_RESET_N_BASE, 0x00, 0x01);

	printf("===== DDR3 Test! Size=A:%dMB/B:%dMB=====\r\n", EMIF_DDR3_ARCH_SPAN/1024/1024, EMIF_DDR3_B_ARCH_SPAN/1024/1024);
	Status = IORD(DDR3_STATUS_BASE, 0x00);

	while(1){
        printf("\n==========================================================\n");
        printf("Press any BUTTON on the board to start test [BUTTON0 for continued test] \n");
        ButtonStatus = ButtonMask;
        while((ButtonStatus & ButtonMask) == ButtonMask){
        	ButtonStatus = IORD(BUTTON_BASE, 0);
        	//ButtonStatus = 0x01;
        }

        if ((ButtonStatus & 0x01) == 0x00){
            bLoop = TRUE;
        }else{
            bLoop = FALSE;
            usleep(300*1000);
        }

		//
        bPassA = TRUE;
        bPassB = TRUE;
        TestIndex = 0;

        do{
        	TestIndex++;
        	printf("=====> DDR3 Testing, Iteration: %d\n", TestIndex);

        	// reset
        	IOWR(REQEUST_DDR3_RESET_N_BASE, 0x00, 0x00);
        	usleep(200*1000);
        	IOWR(REQEUST_DDR3_RESET_N_BASE, 0x00, 0x01);
        	TimeStart = alt_nticks();
        	bTimeout = FALSE;
        	bDone = FALSE;
        	do{
        		Status = IORD(DDR3_STATUS_BASE, 0x00);
        		if (((Status & A_BIT_MASK) != 0) && ((Status & B_BIT_MASK) != 0))
					bDone = TRUE;
				else if ((alt_nticks()-TimeStart) > CalTimeout)
        			bTimeout = TRUE;
        	}while(!bDone && !bTimeout);

        	if (bTimeout)
        		printf("A/B Calibration Timeout, Status = %xh\r\n", Status);
        	else
        		printf("A/B Calibration Duration:%.3f seconds, status=%xh\r\n", (float)(alt_nticks()-TimeStart)/(float)alt_ticks_per_second(), Status);

        	////////////////////////////
        	// ddr3-a
        	TimeStart = alt_nticks();
        	printf("== DDR3-A Testing...\r\n");
        	if (((Status & STATUS_BIT_FAIL) == STATUS_BIT_FAIL) || (((Status & STATUS_BIT_SUCCESS) != STATUS_BIT_SUCCESS))){
        		printf("local init done: fail(Status[local_fail,local_pass]=%xh)\r\n", Status & A_BIT_MASK);
        		bPassA = FALSE;
        	}else{
        		//printf("status=%xh\r\n", Status);
        	}


        	if (bPassA)
        		bPassA = TMEM_Verify(EMIF_DDR3_ARCH_BASE, EMIF_DDR3_ARCH_SPAN, InitValue,  bShowMessage);

        	TimeElapsed = alt_nticks() - TimeStart;
        	printf("DDR3 test:%s, %d seconds\r\n", bPassA?"Pass":"NG", (int)(TimeElapsed/alt_ticks_per_second()));

        	////////////////////////////
        	// ddr3-b
        	TimeStart = alt_nticks();
        	printf("== DDR3-B Testing...\r\n");
        	if (((Status & B_STATUS_BIT_FAIL) == B_STATUS_BIT_FAIL) || (((Status & B_STATUS_BIT_SUCCESS) != B_STATUS_BIT_SUCCESS))){
        		printf("local init done: fail(Status[local_fail,local_pass]=%xh)\r\n", (Status & B_BIT_MASK) >> 2);
        		bPassB = FALSE;
        	}else{
        		//printf("status=%xh\r\n", Status);
        	}


        	if (bPassB)
        		bPassB = TMEM_Verify(EMIF_DDR3_B_ARCH_BASE, EMIF_DDR3_B_ARCH_SPAN, InitValue,  bShowMessage);

        	TimeElapsed = alt_nticks() - TimeStart;
        	printf("DDR3 test:%s, %d seconds\r\n", bPassB?"Pass":"NG", (int)(TimeElapsed/alt_ticks_per_second()));




            if (bPassA && bPassB && bLoop){  // is abort loop?
            	ButtonStatus = IORD(BUTTON_BASE, 0);
            	if ((ButtonStatus & ButtonMask) != ButtonMask)
            		bLoop = FALSE; // press any key to abort continued test
            }

        }while(bLoop && bPassA && bPassB);
	} // while(1)

    return 0;
}


