module Test_QDRIIX1(avl_w_ready,avl_r_ready,avl_r_rdata_valid,avl_r_rdata,afi_clk,afi_reset_n,
						  avl_w_write_req,avl_w_addr,avl_w_size,avl_w_wdata,avl_w_be,avl_r_read_req,avl_r_addr,avl_r_size,
						  local_init_done,local_cal_success,local_cal_fail,test_reset_n,test_pass,test_done
							);

parameter      ADDR_W             =     20;
parameter      DATA_W             =     72;

`define MAX_ADDR	{ADDR_W{1'b1}}//20'hFFFFF

parameter  LFSR_SEED = 36'b000000111110000011110000111000110010;


output								avl_w_write_req;
input         						avl_w_ready;  
output   			 [ADDR_W-1:0] 		avl_w_addr;               
output   			 [2:0]  		avl_w_size;               
output   			 [DATA_W-1:0] 	   avl_w_wdata;             
output   			 [7:0]      avl_w_be;                  
output          					avl_r_read_req;           
input         						avl_r_ready;            
output   			[ADDR_W-1:0] 		avl_r_addr;               
output   			[2:0]  		avl_r_size;                
input         						avl_r_rdata_valid;         
input  				[DATA_W-1:0] 		avl_r_rdata;    
input 								afi_clk;
input 								afi_reset_n;

input									local_init_done;
input									local_cal_success;
input									local_cal_fail;					

input									test_reset_n;		

output 								test_done;
output 								test_pass;				

//////////////////////////////////////////////////


// Pseudo-random data generator
lfsr_wrapper read_data_gen_inst (
	.clk		(afi_clk),
	.reset_n	(test_reset_n),
	.enable	(avl_r_rdata_valid & start_read),
	.data		(avl_expected_data)
	);
defparam read_data_gen_inst.DATA_WIDTH 	= DATA_W;
defparam read_data_gen_inst.SEED       	= LFSR_SEED;

	
// Pseudo-random data generator
lfsr_wrapper write_data_gen_inst (
	.clk		(afi_clk),
	.reset_n	(test_reset_n),
	.enable	(avl_w_write_req),
	.data		(avl_w_wdata)
	);
defparam write_data_gen_inst.DATA_WIDTH	= DATA_W;
defparam write_data_gen_inst.SEED       	= LFSR_SEED;
	
	
//////////////////////////////////////////////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
// memory write



		reg        avl_w_write_req;           //            avl_w.write
		wire       avl_w_ready;               //                 .waitrequest_n
		reg [ADDR_W-1:0] avl_w_addr;                //                 .address
		wire [2:0] avl_w_size;                //                 .burstcount
		wire [DATA_W-1:0] avl_w_wdata;               //                 .writedata
		wire [7:0] avl_w_be;                  //                 .byteenable

		
assign avl_w_be = 8'hFF;
assign avl_w_size = 1;
	
	
reg write_done;	
wire last_write;

assign last_write = (avl_w_write_req && (avl_w_addr == `MAX_ADDR) && avl_w_ready)?1'b1:1'b0;


// write_done
always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
		write_done <= 1'b0;
	else if (~write_done & last_write)
		write_done <= 1'b1;
end
	


// avl_w_write_req
always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
		avl_w_write_req <= 1'b0;
	else if (write_done || last_write)
		avl_w_write_req <= 1'b0;
	else
		avl_w_write_req = 1'b1;
end


// avl_w_addr
always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
		avl_w_addr <= 0;
	else if (~write_done & avl_w_write_req & avl_w_ready)
	begin
		if (avl_w_addr < `MAX_ADDR)
			avl_w_addr <= avl_w_addr + 1;
	end
end




//////////////////////////////////////////////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
// memory read & verify

		reg        avl_r_read_req;            //            avl_r.read
		wire       avl_r_ready;               //                 .waitrequest_n
		reg [ADDR_W-1:0] avl_r_addr;                //                 .address
		wire [2:0] avl_r_size;                //                 .burstcount
		wire       avl_r_rdata_valid;         //                 .readdatavalid
		wire [DATA_W-1:0] avl_r_rdata;               //                 .readdata
		
assign avl_r_size = 1; // chip 4-word burst = 4x18=72 bits.

////////////////////////////
// read start
// idle some cylce to make sure pending read is done
reg			start_read;
reg	[4:0] read_prepare_cnt;
always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
	begin
		read_prepare_cnt <= 0;
		start_read <= 1'b0;
	end 
	else if (!start_read)
	begin
		if (read_prepare_cnt < 5'h1F)
			read_prepare_cnt <= read_prepare_cnt + 1;
		else
			start_read <= 1'b1;
	end
end


////////////////////////////
// pending count
reg   [4:0] read_pending_cnt;
`define MAX_READ_PENDING_NUM	16

always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
		read_pending_cnt <= 0;
	else if (avl_r_read_req & ~avl_r_rdata_valid)
		read_pending_cnt <= read_pending_cnt + 1;
	else if (~avl_r_read_req & avl_r_rdata_valid)
		read_pending_cnt <= read_pending_cnt - 1;
end

////////////////////////////
// issue read request signal: avl_r_read_req

wire can_issue_read_request;
assign can_issue_read_request = (start_read && (read_pending_cnt < `MAX_READ_PENDING_NUM) && avl_r_ready && (avl_r_addr < avl_w_addr || write_done) && ~last_read_request_done) ?1'b1:1'b0;

always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
		avl_r_read_req <= 1'b0;
	else if (can_issue_read_request)
		avl_r_read_req <= 1'b1;
	else if (avl_r_read_req)	
		avl_r_read_req <= 1'b0;
end

////////////////////////////
// avl_r_addr
always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
		avl_r_addr <= 0;
	else if (avl_r_read_req && avl_r_addr != `MAX_ADDR && ~last_read_request_done)
		avl_r_addr <= avl_r_addr + 1;
end

////////////////////////////
// last_read_request_done
reg last_read_request_done;
always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
		last_read_request_done <= 1'b0;
	else if (avl_r_read_req && avl_r_addr == `MAX_ADDR)
		last_read_request_done <= 1'b1;
end


////////////////////////////
// verify
wire [DATA_W-1:0] avl_expected_data;               
reg [ADDR_W-1:0] avl_read_cnt;      
reg verify_ng;
reg read_done;

always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
	begin
		verify_ng <= 1'b0;
		//avl_expected_data <= 0;
		avl_read_cnt <= 0;
		read_done <= 1'b0;
	end
	else if (~read_done && avl_r_rdata_valid)
	begin
		if (avl_r_rdata != avl_expected_data)
			verify_ng <= 1'b1;
		
	//	avl_expected_data <= avl_expected_data + 1;
		if (avl_read_cnt == `MAX_ADDR)
			read_done <= 1'b1;
		else
			avl_read_cnt <= avl_read_cnt + 1;
	end
end

//////////////////////////////////////////////////
// test status & result indicaiton
reg test_done;
reg test_pass;


always @ (posedge afi_clk or negedge test_reset_n)
begin
	if (~test_reset_n)
	begin
		if (local_init_done && local_cal_success)
		begin
			test_done <= 1'b0;
			test_pass <= 1'b1;
		end
		else
		begin
			test_done <= 1'b1; 
			test_pass <= 1'b0; // fail
		end
	end
	else if (~test_done)
	begin
		test_done <= write_done & read_done;
		
		if (verify_ng)
			test_pass <= 1'b0;
	end
end
	
							
endmodule 